/****
*
*      oInt.prg
*      oObject Class Demonstration.
*
*      Notes:
*      ------
*      An all purpose interrupt handler based on FT_INT86() (from Nanfor.lib)
*      This class is very flexible allowing you to modify both high and low
*      registers (e.g. AL and AH as well as AX) without arithmetics.
*
*      The Test() proc sets the mode in 320x200x256, outputs some pixels
*      and returns you to text mode. See how you can modify AL & AH together
*      with CX and DX. Also, you can get the values returned to the registers
*      in exactly the same way. Then it creates a DOS file and returns the
*      current directory.
*
*      For more information please read the FT_INT86() documentation. This
*      class is only a shell for this great function written by Ted Means.
*
*      Note:
*      Atilla Szabo noticed that some INT21 functions don't work properly under
*      Win95 OSR2. Thanks Atilla...
*/

#include "oObject.ch"
#include "oInt.ch"


//#ifdef TEST
//
Proc TestInt()
//
Local i
Local o := oInt():New()
o:ClassInfo()
// --> Output some pixels in grafx
o:AH := 0
o:AL := 19  // 13h
o:Exec( o:IntVideo )    // set 320x200x256 mode
o:Reset()                  // reset interrupts to 0

for i=1 to 100
     o:AH := 12  // 0Ch put pixel on screen
     o:AL := i   // color of pixel
     o:CX := i   // column (X)
     o:DX := i   // row (Y)
     o:Exec( o:IntVideo )
     o:Reset()
NEXT
Inkey(0)

o:AH := 0
o:AL := 3  // 03h  Text Mode
o:Exec( o:IntVideo )    // set text mode

o:Reset()

// --> Get the current directory
// --> Calling conventions are the same as in FT_INT86()
o:AH := 71
o:DX := 0
o:DS := space(64)
o:SI := REG_DS
o:Exec( o:intDOS )

?
? "Current Directory :", o:DS
?
? "o:ClassInfo() contains now the return values..."
? "Press a key..."
inkey(0)
o:classInfo()

// --> Create a DOS file

FErase("INT_FILE.TMP")

o:AH := 60                    // function 0x3C "Create file handle"
o:CX := 0                     // attribute
o:DS := "INT_FILE.TMP"        // ASCIZ filename to create
o:DX := REG_DS                // Points to DS:DX
o:Exec( 33 )
? o:AX

IF file("INT_FILE.TMP")
   ? "File  '"+"INT_FILE.TMP"+"' created..."
ELSE
   ? "ERROR: File  '"+"INT_FILE.TMP"+"' NOT created..."
endif

o:Reset()

Return

//#endif


//
CLASS oInt
//
READONLY:
// Most common interrupt numbers. These are ReadOnly and can not be modified
DATA intVideo    AS 16  // x10
DATA intEquip    AS 17  // x11
DATA intDisk     AS 19  // x13
DATA intSerial   AS 20  // x14
DATA intKeyBoard AS 22  // x16
DATA intPrinter  AS 23  // x17
DATA intClock    AS 26  // x1A
DATA intDos      AS 33  // x21
DATA intMouse    AS 51  // x33

/*
You could also do it this way... so oInt:IntVideo() would
perform an ::Exec(16) directly - if all registers were set.

DATA intVideo    INLINE ::Exec( 16 )
DATA intEquip    INLINE ::Exec( 17 )
DATA intDisk     INLINE ::Exec( 19 )
DATA intSerial   INLINE ::Exec( 20 )
DATA intKeyBoard INLINE ::Exec( 22 )
DATA intPrinter  INLINE ::Exec( 23 )
DATA intClock    INLINE ::Exec( 26 )
DATA intDos      INLINE ::Exec( 33 )
DATA intMouse    INLINE ::Exec( 51 )

*/

EXPORT:

DATA AX, AH, AL
DATA BX, BH, BL
DATA CX, CH, CL
DATA DX, DH, DL
DATA SI
DATA DI
DATA BP
DATA DS
DATA ES

METHOD New()
METHOD Reset()
METHOD Exec( nInt )

ENDCLASS



//
METHOD New()
//
::Reset()
Return( self )


// Resets internal variables setting all values to 0
//
METHOD Reset()
//
::AX := 0 ; ::AH := 0 ; ::AL := 0
::BX := 0 ; ::BH := 0 ; ::BL := 0
::CX := 0 ; ::CH := 0 ; ::CL := 0
::DX := 0 ; ::DH := 0 ; ::DL := 0
::SI := 0 ; ::DI := 0 ; ::BP := 0
::DS := 0 ; ::ES := 0
Return( self )



// Executes an interrupt. Uses the values in the Instance Variables to
// build an array to feed FT_INT86(), then sets the instance Variables
// to the values returned by FT_INT86()
//
METHOD Exec( nInt )
//
Local aRegs[10]

aRegs[_AX] := IF( !Valtype(::AX)=="L", IF(::AX=0 .OR. ::AX=NIL, (::AH*256 + ::AL), ::AX ), ::AX )
aRegs[_BX] := IF( !Valtype(::BX)=="L", IF(::BX=0 .OR. ::BX=NIL, (::BH*256 + ::BL), ::BX ), ::BX )
aRegs[_CX] := IF( !Valtype(::CX)=="L", IF(::CX=0 .OR. ::CX=NIL, (::CH*256 + ::CL), ::CX ), ::CX )
aRegs[_DX] := IF( !Valtype(::DX)=="L", IF(::DX=0 .OR. ::DX=NIL, (::DH*256 + ::DL), ::DX ), ::DX )
aRegs[_SI] := ::SI
aRegs[_DI] := ::DI
aRegs[_BP] := ::BP
aRegs[_DS] := ::DS
aRegs[_ES] := ::ES

FT_INT86( nInt, aRegs )

::AX := aRegs[_AX] ; ::AL := LowByte( ::AX ) ; ::AH := HighByte( ::AX )
::BX := aRegs[_BX] ; ::BL := LowByte( ::BX ) ; ::BH := HighByte( ::BX )
::CX := aRegs[_CX] ; ::CL := LowByte( ::CX ) ; ::CH := HighByte( ::CX )
::DX := aRegs[_DX] ; ::DL := LowByte( ::DX ) ; ::DH := HighByte( ::DX )
::SI := aRegs[_SI]
::DI := aRegs[_DI]
::BP := aRegs[_BP]
::DS := aRegs[_DS]
::ES := aRegs[_ES]

Return( self )


